@@ -66,10 +66,11 @@ class ComponentConfig(object):
         'extra',
         'keyring',
         'controller',
-        'shared'
+        'shared',
+        'runner',
     )
 
-    def __init__(self, realm=None, extra=None, keyring=None, controller=None, shared=None):
+    def __init__(self, realm=None, extra=None, keyring=None, controller=None, shared=None, runner=None):
         """
 
         :param realm: The realm the session would like to join or ``None`` to let the router
@@ -101,6 +102,9 @@ class ComponentConfig(object):
             this feature can introduce coupling between components. A valid use case would be
             to hold a shared database connection pool.
         :type shared: dict or None
+
+        :param runner: Instance of ApplicationRunner when run under this.
+        :type runner: :class:`autobahn.twisted.wamp.ApplicationRunner`
         """
         assert(realm is None or type(realm) == six.text_type)
         # assert(keyring is None or ...) # FIXME
@@ -110,9 +114,10 @@ class ComponentConfig(object):
         self.keyring = keyring
         self.controller = controller
         self.shared = shared
+        self.runner = runner
 
     def __str__(self):
-        return u"ComponentConfig(realm=<{}>, extra={}, keyring={}, controller={}, shared={})".format(self.realm, self.extra, self.keyring, self.controller, self.shared)
+        return u"ComponentConfig(realm=<{}>, extra={}, keyring={}, controller={}, shared={}, runner={})".format(self.realm, self.extra, self.keyring, self.controller, self.shared, self.runner)
 
 
 @public
@@ -376,6 +381,21 @@ class SessionDetails(object):
         self.resumable = resumable
         self.resume_token = resume_token
 
+    def marshal(self):
+        obj = {
+            u'realm': self.realm,
+            u'session': self.session,
+            u'authid': self.authid,
+            u'authrole': self.authrole,
+            u'authmethod': self.authmethod,
+            u'authprovider': self.authprovider,
+            u'authextra': self.authextra,
+            u'resumed': self.resumed,
+            u'resumable': self.resumable,
+            u'resume_token': self.resume_token
+        }
+        return obj
+
     def __str__(self):
         return u"SessionDetails(realm=<{}>, session={}, authid=<{}>, authrole=<{}>, authmethod={}, authprovider={}, authextra={}, resumed={}, resumable={}, resume_token={})".format(self.realm, self.session, self.authid, self.authrole, self.authmethod, self.authprovider, self.authextra, self.resumed, self.resumable, self.resume_token)
 
@@ -427,9 +447,9 @@ class SessionIdent(object):
 
     def marshal(self):
         obj = {
-            'session': self.session,
-            'authid': self.authid,
-            'authrole': self.authrole,
+            u'session': self.session,
+            u'authid': self.authid,
+            u'authrole': self.authrole,
         }
         return obj
 
@@ -514,6 +534,13 @@ class CloseDetails(object):
         self.reason = reason
         self.message = message
 
+    def marshal(self):
+        obj = {
+            u'reason': self.reason,
+            u'message': self.message
+        }
+        return obj
+
     def __str__(self):
         return u"CloseDetails(reason=<{}>, message='{}')".format(self.reason, self.message)
 
@@ -530,13 +557,14 @@ class SubscribeOptions(object):
         'details',
         'details_arg',
         'get_retained',
+        'forward_for',
         'correlation_id',
         'correlation_uri',
         'correlation_is_anchor',
         'correlation_is_last',
     )
 
-    def __init__(self, match=None, details=None, details_arg=None, get_retained=None,
+    def __init__(self, match=None, details=None, details_arg=None, forward_for=None, get_retained=None,
                  correlation_id=None, correlation_uri=None, correlation_is_anchor=None,
                  correlation_is_last=None):
         """
@@ -560,16 +588,25 @@ class SubscribeOptions(object):
         assert(details_arg is None or type(details_arg) == str)  # yes, "str" is correct here, since this is about Python identifiers!
         assert(get_retained is None or type(get_retained) is bool)
 
+        assert(forward_for is None or type(forward_for) == list)
+        if forward_for:
+            for ff in forward_for:
+                assert type(ff) == dict
+                assert 'session' in ff and type(ff['session']) in six.integer_types
+                assert 'authid' in ff and (ff['authid'] is None or type(ff['authid']) == six.text_type)
+                assert 'authrole' in ff and type(ff['authrole']) == six.text_type
+
         self.match = match
-        self.details = details
 
         # FIXME: this is for backwards compat, but we'll deprecate it in the future
+        self.details = details
         if details:
             self.details_arg = 'details'
         else:
             self.details_arg = details_arg
 
         self.get_retained = get_retained
+        self.forward_for = forward_for
 
         self.correlation_id = correlation_id
         self.correlation_uri = correlation_uri
@@ -588,10 +625,13 @@ class SubscribeOptions(object):
         if self.get_retained is not None:
             options[u'get_retained'] = self.get_retained
 
+        if self.forward_for is not None:
+            options[u'forward_for'] = self.forward_for
+
         return options
 
     def __str__(self):
-        return u"SubscribeOptions(match={}, details={}, details_arg={}, get_retained={})".format(self.match, self.details, self.details_arg, self.get_retained)
+        return u"SubscribeOptions(match={}, details={}, details_arg={}, get_retained={}, forward_for={})".format(self.match, self.details, self.details_arg, self.get_retained, self.forward_for)
 
 
 @public
@@ -662,7 +702,7 @@ class EventDetails(object):
             for ff in forward_for:
                 assert type(ff) == dict
                 assert 'session' in ff and type(ff['session']) in six.integer_types
-                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authid' in ff and (ff['authid'] is None or type(ff['authid']) == six.text_type)
                 assert 'authrole' in ff and type(ff['authrole']) == six.text_type
 
         self.subscription = subscription
@@ -762,13 +802,16 @@ class PublishOptions(object):
         assert(eligible_authrole is None or type(eligible_authrole) == six.text_type or (type(eligible_authrole) == list and all(type(x) == six.text_type for x in eligible_authrole)))
         assert(retain is None or type(retain) == bool)
 
-        assert(forward_for is None or type(forward_for) == list)
+        assert(forward_for is None or type(forward_for) == list), 'forward_for, when present, must have list type - was {}'.format(type(forward_for))
         if forward_for:
             for ff in forward_for:
-                assert type(ff) == dict
-                assert 'session' in ff and type(ff['session']) in six.integer_types
-                assert 'authid' in ff and type(ff['authid']) == six.text_type
-                assert 'authrole' in ff and type(ff['authrole']) == six.text_type
+                assert type(ff) == dict, 'forward_for must be type dict - was {}'.format(type(ff))
+                assert 'session' in ff, 'forward_for must have session attribute'
+                assert type(ff['session']) in six.integer_types, 'forward_for.session must have integer type - was {}'.format(type(ff['session']))
+                assert 'authid' in ff, 'forward_for must have authid attributed'
+                assert type(ff['authid']) == six.text_type, 'forward_for.authid must have str type - was {}'.format(type(ff['authid']))
+                assert 'authrole' in ff, 'forward_for must have authrole attribute'
+                assert type(ff['authrole']) == six.text_type, 'forward_for.authrole must have str type - was {}'.format(type(ff['authrole']))
 
         self.acknowledge = acknowledge
         self.exclude_me = exclude_me
@@ -840,6 +883,8 @@ class RegisterOptions(object):
         'invoke',
         'concurrency',
         'force_reregister',
+        'forward_for',
+        'details',
         'details_arg',
         'correlation_id',
         'correlation_uri',
@@ -847,9 +892,9 @@ class RegisterOptions(object):
         'correlation_is_last',
     )
 
-    def __init__(self, match=None, invoke=None, concurrency=None, details_arg=None, force_reregister=None,
-                 correlation_id=None, correlation_uri=None, correlation_is_anchor=None,
-                 correlation_is_last=None):
+    def __init__(self, match=None, invoke=None, concurrency=None, force_reregister=None, forward_for=None,
+                 details=None, details_arg=None, correlation_id=None, correlation_uri=None,
+                 correlation_is_anchor=None, correlation_is_last=None):
         """
         :param match: Type of matching to use on the URI (`exact`, `prefix` or `wildcard`)
 
@@ -864,23 +909,51 @@ class RegisterOptions(object):
             in this keyword argument to the callable.
         :type details_arg: str
 
+        :param details: When invoking the endpoint, provide call details in a keyword
+            parameter ``details``.
+        :type details: bool
+
+        :param details_arg: DEPCREATED (use "details" flag). When invoking the endpoint,
+            provide call details in this keyword argument to the callable.
+        :type details_arg: str
+
         :param force_reregister: if True, any other session that has
             already registered this URI will be 'kicked out' and this
             session will become the one that's registered (the previous
             session must have used `force_reregister=True` as well)
+        :type force_reregister: bool
 
+        :param forward_for: When this Register is forwarded over a router-to-router link,
+            or via an intermediary router.
+        :type forward_for: list[dict]
         """
         assert(match is None or (type(match) == six.text_type and match in [u'exact', u'prefix', u'wildcard']))
         assert(invoke is None or (type(invoke) == six.text_type and invoke in [u'single', u'first', u'last', u'roundrobin', u'random']))
         assert(concurrency is None or (type(concurrency) in six.integer_types and concurrency > 0))
+        assert(details is None or (type(details) == bool and details_arg is None))
         assert(details_arg is None or type(details_arg) == str)  # yes, "str" is correct here, since this is about Python identifiers!
         assert force_reregister in [None, True, False]
 
+        assert(forward_for is None or type(forward_for) == list)
+        if forward_for:
+            for ff in forward_for:
+                assert type(ff) == dict
+                assert 'session' in ff and type(ff['session']) in six.integer_types
+                assert 'authid' in ff and (ff['authid'] is None or type(ff['authid']) == six.text_type)
+                assert 'authrole' in ff and type(ff['authrole']) == six.text_type
+
         self.match = match
         self.invoke = invoke
         self.concurrency = concurrency
-        self.details_arg = details_arg
         self.force_reregister = force_reregister
+        self.forward_for = forward_for
+
+        # FIXME: this is for backwards compat, but we'll deprecate it in the future
+        self.details = details
+        if details:
+            self.details_arg = 'details'
+        else:
+            self.details_arg = details_arg
 
         self.correlation_id = correlation_id
         self.correlation_uri = correlation_uri
@@ -905,10 +978,13 @@ class RegisterOptions(object):
         if self.force_reregister is not None:
             options[u'force_reregister'] = self.force_reregister
 
+        if self.forward_for is not None:
+            options[u'forward_for'] = self.forward_for
+
         return options
 
     def __str__(self):
-        return u"RegisterOptions(match={}, invoke={}, concurrency={}, details_arg={}, force_reregister={})".format(self.match, self.invoke, self.concurrency, self.details_arg, self.force_reregister)
+        return u"RegisterOptions(match={}, invoke={}, concurrency={}, details={}, details_arg={}, force_reregister={}, forward_for={})".format(self.match, self.invoke, self.concurrency, self.details, self.details_arg, self.force_reregister, self.forward_for)
 
 
 @public
@@ -974,7 +1050,7 @@ class CallDetails(object):
             for ff in forward_for:
                 assert type(ff) == dict
                 assert 'session' in ff and type(ff['session']) in six.integer_types
-                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authid' in ff and (ff['authid'] is None or type(ff['authid']) == six.text_type)
                 assert 'authrole' in ff and type(ff['authrole']) == six.text_type
 
         self.registration = registration
@@ -999,21 +1075,29 @@ class CallOptions(object):
     __slots__ = (
         'on_progress',
         'timeout',
+        'caller',
+        'caller_authid',
+        'caller_authrole',
         'forward_for',
         'correlation_id',
         'correlation_uri',
         'correlation_is_anchor',
         'correlation_is_last',
+        'details',
     )
 
     def __init__(self,
                  on_progress=None,
                  timeout=None,
+                 caller=None,
+                 caller_authid=None,
+                 caller_authrole=None,
                  forward_for=None,
                  correlation_id=None,
                  correlation_uri=None,
                  correlation_is_anchor=None,
-                 correlation_is_last=None):
+                 correlation_is_last=None,
+                 details=None):
         """
 
         :param on_progress: A callback that will be called when the remote endpoint
@@ -1028,19 +1112,27 @@ class CallOptions(object):
         """
         assert(on_progress is None or callable(on_progress))
         assert(timeout is None or (type(timeout) in list(six.integer_types) + [float] and timeout > 0))
-
+        assert(details is None or type(details) == bool)
+        assert(caller is None or type(caller) in six.integer_types)
+        assert(caller_authid is None or type(caller_authid) == six.text_type)
+        assert(caller_authrole is None or type(caller_authrole) == six.text_type)
         assert(forward_for is None or type(forward_for) == list)
         if forward_for:
             for ff in forward_for:
                 assert type(ff) == dict
                 assert 'session' in ff and type(ff['session']) in six.integer_types
-                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authid' in ff and (ff['authid'] is None or type(ff['authid']) == six.text_type)
                 assert 'authrole' in ff and type(ff['authrole']) == six.text_type
 
         self.on_progress = on_progress
         self.timeout = timeout
+
+        self.caller = caller
+        self.caller_authid = caller_authid
+        self.caller_authrole = caller_authrole
         self.forward_for = forward_for
 
+        self.details = details
         self.correlation_id = correlation_id
         self.correlation_uri = correlation_uri
         self.correlation_is_anchor = correlation_is_anchor
@@ -1052,6 +1144,9 @@ class CallOptions(object):
         """
         options = {}
 
+        # note: only some attributes are actually forwarded to the WAMP CALL message, while
+        # other attributes are for client-side/client-internal use only
+
         if self.timeout is not None:
             options[u'timeout'] = self.timeout
 
@@ -1061,10 +1156,19 @@ class CallOptions(object):
         if self.forward_for is not None:
             options[u'forward_for'] = self.forward_for
 
+        if self.caller is not None:
+            options[u'caller'] = self.caller
+
+        if self.caller_authid is not None:
+            options[u'caller_authid'] = self.caller_authid
+
+        if self.caller_authrole is not None:
+            options[u'caller_authrole'] = self.caller_authrole
+
         return options
 
     def __str__(self):
-        return u"CallOptions(on_progress={}, timeout={}, forward_for={})".format(self.on_progress, self.timeout, self.forward_for)
+        return u"CallOptions(on_progress={}, timeout={}, caller={}, caller_authid={}, caller_authrole={}, forward_for={}, details={})".format(self.on_progress, self.timeout, self.caller, self.caller_authid, self.caller_authrole, self.forward_for, self.details)
 
 
 @public
@@ -1078,6 +1182,9 @@ class CallResult(object):
         'results',
         'kwresults',
         'enc_algo',
+        'callee',
+        'callee_authid',
+        'callee_authrole',
         'forward_for',
     )
 
@@ -1093,23 +1200,33 @@ class CallResult(object):
         enc_algo = kwresults.pop('enc_algo', None)
         assert(enc_algo is None or type(enc_algo) == six.text_type)
 
+        callee = kwresults.pop('callee', None)
+        callee_authid = kwresults.pop('callee_authid', None)
+        callee_authrole = kwresults.pop('callee_authrole', None)
+
+        assert callee is None or type(callee) in six.integer_types
+        assert callee_authid is None or type(callee_authid) == six.text_type
+        assert callee_authrole is None or type(callee_authrole) == six.text_type
+
         forward_for = kwresults.pop('forward_for', None)
         assert(forward_for is None or type(forward_for) == list)
         if forward_for:
             for ff in forward_for:
                 assert type(ff) == dict
                 assert 'session' in ff and type(ff['session']) in six.integer_types
-                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authid' in ff and (ff['authid'] is None or type(ff['authid']) == six.text_type)
                 assert 'authrole' in ff and type(ff['authrole']) == six.text_type
 
         self.enc_algo = enc_algo
+        self.callee = callee
+        self.callee_authid = callee_authid
+        self.callee_authrole = callee_authrole
         self.forward_for = forward_for
-
         self.results = results
         self.kwresults = kwresults
 
     def __str__(self):
-        return u"CallResult(results={}, kwresults={}, enc_algo={}, forward_for={})".format(self.results, self.kwresults, self.enc_algo, self.forward_for)
+        return u"CallResult(results={}, kwresults={}, enc_algo={}, callee={}, callee_authid={}, callee_authrole={}, forward_for={})".format(self.results, self.kwresults, self.enc_algo, self.callee, self.callee_authid, self.callee_authrole, self.forward_for)
 
 
 @public
